<?php

namespace Drupal\statsd_client\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;

/**
 * Subscribe to KernelEvents::REQUEST and ::TERMINATE events.
 */
class StatsdEventSubscriber implements EventSubscriberInterface {

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a StatsdEventSubscriber object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   The config.
   * @param \Drupal\Core\Database\Connection $database
   *   The database.
   */
  public function __construct(ConfigFactoryInterface $config, Connection $database) {
    $this->config = $config->get('statsd_client.settings');
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Setting low priority to go early in the event stack.
    $events[KernelEvents::REQUEST][] = ['statsdBootHandler', -100];

    // Setting high priority to go late in the event stack.
    $events[KernelEvents::TERMINATE][] = ['statsdTerminateHandler', 100];
    return $events;
  }

  /**
   * Set a static variable to support request processing time tracking.
   */
  public function statsdBootHandler() {
    if ($this->config->get('events.performance_events')) {
      drupal_static('statsd_timer', microtime(TRUE));
    }
  }

  /**
   * Set configured metrics on shut down.
   */
  public function statsdTerminateHandler() {
    if ($this->config->get('events.user_events')) {
      $active_sessions = $this->database->query("SELECT count(*) as num FROM {sessions} WHERE timestamp > UNIX_TIMESTAMP() - 3600")->fetchField();
      \Drupal::service('statsd_client.manager')->gauge('user_events.active_sessions', $active_sessions);
      \Drupal::service('statsd_client.manager')->increment('user_events.page_view');
    }

    if ($this->config->get('events.performance_events')) {
      $memory = round(memory_get_peak_usage() / 1024 / 1024, 2);
      \Drupal::service('statsd_client.manager')->gauge('performance_events.peak_memory', $memory);

      $start = &drupal_static('statsd_timer');
      // hook_boot() may not be called in certain contexts.
      if ($start > 0) {
        $end = microtime(TRUE);
        $time = round(($end - $start) * 1000, 0);
        \Drupal::service('statsd_client.manager')->timing('performance_events.execution_time', $time);
      }
    }
  }

}
